<?php

namespace Pinpoint\Conesso\Model\Connector;

use Magento\Directory\Api\CountryInformationAcquirerInterface;
use Pinpoint\Conesso\Model\Product\AttributeFactory;

/**
 * Transactional data for orders, including mapped custom order attributes to sync.
 *
 * @SuppressWarnings(PHPMD.TooManyFields)
 */
class Order
{
    /**
     * Order Increment ID.
     *
     * @var string
     */
    public $id;

    /**
     * Email.
     *
     * @var string
     */
    public $email;

    /**
     * @var integer
     */
    public $quoteId;

    /**
     * @var integer
     */
    public $customerId;

    /**
     * @var string
     */
    public $storeName;

    /**
     * @var string
     */
    public $purchaseDate;

    /**
     * @var array
     */
    public $deliveryAddress = [];

    /**
     * @var array
     */
    public $billingAddress = [];

    /**
     * @var array
     */
    public $products = [];

    /**
     * @var float
     */
    public $orderSubtotal;

    /**
     * @var float
     */
    public $discountAmount;

    /**
     * @var float
     */
    public $orderTotal;

    /**
     * Payment name.
     *
     * @var string
     */
    public $payment;

    /**
     * @var string
     */
    public $deliveryMethod;

    /**
     * @var float
     */
    public $deliveryTotal;

    /**
     * @var string
     */
    public $currency;

    /**
     * @var object
     */
    public $couponCode;

    /**
     * @var array
     */
    public $custom = [];

    /**
     * @var string
     */
    public $orderStatus;

    /**
     * @var \Pinpoint\Conesso\Helper\Data
     */
    public $helper;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    private $storeManager;

    /**
     * @var \Magento\Customer\Model\CustomerFactory
     */
    public $customerFactory;

    /**
     * @var \Magento\Catalog\Model\ProductFactory
     */
    public $productFactory;

    /**
     * @var KeyValidator
     */
    private $validator;

    /**
     * @var AttributeFactory $attributeHandler
     */
    private $attributeHandler;

    private $createdAt;

    private $updatedAt;

    private $countryInformationAcquirer;

    /**
     * Order constructor.
     *
     * @param \Magento\Catalog\Model\ProductFactory      $productFactory
     * @param \Magento\Customer\Model\CustomerFactory    $customerFactory
     * @param \Pinpoint\Conesso\Helper\Data            $helperData
     * @param \Magento\Store\Model\StoreManagerInterface $storeManagerInterface
     * @param KeyValidator                               $validator
     * @param AttributeFactory                           $attributeHandler
     * @param CountryInformationAcquirerInterface        $countryInformationAcquirer
     */
    public function __construct(
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \Magento\Customer\Model\CustomerFactory $customerFactory,
        \Pinpoint\Conesso\Helper\Data $helperData,
        \Magento\Store\Model\StoreManagerInterface $storeManagerInterface,
        KeyValidator $validator,
        AttributeFactory $attributeHandler,
        CountryInformationAcquirerInterface $countryInformationAcquirer
    ) {
        $this->productFactory      = $productFactory;
        $this->customerFactory     = $customerFactory;
        $this->helper              = $helperData;
        $this->storeManager = $storeManagerInterface;
        $this->validator = $validator;
        $this->attributeHandler = $attributeHandler;
        $this->countryInformationAcquirer = $countryInformationAcquirer;
    }

    /**
     * Set the order data information.
     *
     * @param \Magento\Sales\Model\Order $orderData
     *
     * @return $this
     */
    public function setOrderData($orderData)
    {
        $this->id = $orderData->getIncrementId();
        $this->api_reference_id = $orderData->getId();
        $this->api_integration = 'Pinpoint_Conesso';
        $this->api_resource = 'order';
        $this->order_create_date = gmdate("Y-m-d\TH:i:s.\\0\\0\\0\\Z", strtotime($orderData->getCreatedAt()));
        $this->order_update_date = gmdate("Y-m-d\TH:i:s.\\0\\0\\0\\Z", strtotime($orderData->getUpdatedAt()));
        $this->order_status = $orderData->getStatus();
        $this->order_reference = $orderData->getIncrementId();

        $store = $orderData->getStore();
        $this->store_id = $orderData->getStoreId();
        $this->store_code = $store->getCode();
        $this->store_name = $orderData->getStoreName();

        $this->invoice_tax_total = (float)$orderData->getTaxAmount();
        $this->invoice_total = (float)$orderData->getData('subtotal');
        $this->invoice_total_inc_tax = (float)$orderData->getSubtotalInclTax();
        $this->invoice_discount_total = (float)$orderData->getDiscountAmount();
        $this->shipping_tax_total = (float)$orderData->getShippingTaxAmount();
        $this->shipping_total = (float)$orderData->getShippingAmount();
        $this->shipping_total_inc_tax = (float)$orderData->getShippingInclTax();
        $this->shipping_discount_total = (float)$orderData->getShippingDiscountAmount();
        $this->grand_total = (float)$orderData->getGrandTotal() - $orderData->getTaxAmount();
        $this->grand_total_inc_tax = (float)$orderData->getGrandTotal();
        $this->iso_currency_code = $orderData->getOrderCurrencyCode();

        $this->payment_type = $orderData->getPayment()->getMethod();

        $payment = $orderData->getPayment();
        if ($payment) {
            if ($payment->getMethod()) {
                $methodInstance = $payment->getMethodInstance($payment->getMethod());
                if ($methodInstance) {
                    $this->payment_type_name = $methodInstance->getTitle();
                }
            }
        }

        $this->number_of_products = count($orderData->getAllVisibleItems());
        $this->customer_email = $orderData->getCustomerEmail();
        $this->customer_firstname = $orderData->getCustomerFirstname();
        $this->customer_lastname = $orderData->getCustomerLastname();
        $this->customer_gender = $orderData->getCustomerGender();
        $this->customer_telephone = '';
        $this->customer_mobile = '';
        $this->customer_dob = $orderData->getCustomerDob();
        $this->customer_is_guest = $orderData->getCustomerId() ? false : true;
        $this->customer_notes = $orderData->getCustomerNote();

        $billing = $orderData->getBillingAddress();
        $billingstreet = $billing->getStreet();

        $this->billing_first_name = $billing->getFirstname();
        $this->billing_last_name = $billing->getLastname();
        $this->billing_address1 = (isset($billingstreet[0]) ? $billingstreet[0] : '');
        $this->billing_address2 = (isset($billingstreet[1]) ? $billingstreet[1] : '');
        $this->billing_city = $billing->getCity();
        $this->billing_county = $billing->getRegion();
        $this->billing_post_code = $billing->getPostcode();
        $countryData = $this->countryInformationAcquirer->getCountryInfo($billing->getCountryId());
        $this->billing_country = $countryData->getFullNameLocale();

        $shipping = $orderData->getShippingAddress();
        $shippingstreet = $shipping->getStreet();

        $this->shipping_first_name = $shipping->getFirstname();
        $this->shipping_last_name = $shipping->getLastname();
        $this->shipping_address1 = (isset($shippingstreet[0]) ? $shippingstreet[0] : '');
        $this->shipping_address2 = (isset($shippingstreet[1]) ? $shippingstreet[1] : '');
        $this->shipping_city = $shipping->getCity();
        $this->shipping_county = $shipping->getRegion();
        $this->shipping_post_code = $shipping->getPostcode();
        $countryData = $this->countryInformationAcquirer->getCountryInfo($shipping->getCountryId());
        $this->shipping_country = $countryData->getFullNameLocale();

        /*
         * custom order attributes
         */
        $website = $this->storeManager->getStore($orderData->getStore())->getWebsite();

        $customAttributes
            = $this->helper->getConfigSelectedCustomOrderAttributes(
                $website
            );

        if ($customAttributes) {
            $fields = $this->helper->getOrderTableDescription();
            $this->custom = [];
            foreach ($customAttributes as $customAttribute) {
                if (isset($fields[$customAttribute])) {
                    $field = $fields[$customAttribute];
                    $value = $this->_getCustomAttributeValue(
                        $field,
                        $orderData
                    );
                    if ($value) {
                        $this->_assignCustom($field, $value);
                    }
                }
            }
        }

        $syncCustomOption = $this->helper->getWebsiteConfig(
            \Pinpoint\Conesso\Helper\Config::XML_PATH_CONNECTOR_SYNC_ORDER_PRODUCT_CUSTOM_OPTIONS,
            $website
        );

        /*
         * Order items.
         */
        $this->processOrderItems($orderData->getIncrementId(), $orderData, $syncCustomOption);

        unset($this->storeManager);
        unset($this->countryInformationAcquirer);

        return $this;
    }

    /**
     * @param \Magento\Sales\Model\Order $orderData
     * @param boolean                    $syncCustomOption
     *
     * @return null
     */
    private function processOrderItems($order_id, $orderData, $syncCustomOption)
    {
        foreach ($orderData->getAllVisibleItems() as $productItem) {
            //product custom options
            $customOptions = [];
            if ($syncCustomOption) {
                $customOptions = $this->_getOrderItemOptions($productItem);
            }

            $productModel = $productItem->getProduct();

            if ($productModel) {
                // category names
                $categoryCollection = $productModel->getCategoryCollection()
                    ->addAttributeToSelect('name');
                $productCat = [];
                foreach ($categoryCollection as $cat) {
                    $categories = [];
                    $categories[] = $cat->getName();
                    $productCat[]['Name'] = mb_substr(
                        implode(', ', $categories),
                        0,
                        \Pinpoint\Conesso\Helper\Data::DM_FIELD_LIMIT
                    );
                }

                $attributeModel = $this->attributeHandler->create();
                $attributes = null;

                //selected attributes from config
                $configAttributes = $attributeModel->getConfigAttributesForSync(
                    $orderData->getStore()->getWebsite()
                );

                if ($configAttributes) {
                    $configAttributes = explode(',', $configAttributes);
                    //attributes from attribute set
                    $attributesFromAttributeSet = $attributeModel->getAttributesArray(
                        $productModel->getAttributeSetId()
                    );

                    $attributes = $attributeModel->processConfigAttributes(
                        $configAttributes,
                        $attributesFromAttributeSet,
                        $productModel
                    );
                }

                $attributeSetName = $attributeModel->getAttributeSetName($productModel);

                $productData = [
                    'id' => $productItem->getProductId(),
                    'order_id' => $order_id,
                    'order_line_id' => $productItem->getItemId(),
                    'product_sku' => $productItem->getSku(),
                    'product_name' => $productItem->getName(),
                    'product_description' => $productItem->getDescription(),
                    'tax_percent' => (float)$productItem->getTaxPercent(),
                    'product_price' => (float)$productItem->getPrice(),
                    'product_price_tax' => (float)$productItem->getTaxAmount(),
                    'product_price_inc_tax' => (float)$productItem->getPriceInclTax(),
                    'product_original_price' => (float)$productItem->getBasePrice(),
                    'quantity' => (float)number_format(
                        $productItem->getData('qty_ordered'),
                        2
                    ),
                    'row_total' => (float)$productItem->getRowTotal(),
                    'row_total_inc_tax' => (float)$productItem->getRowTotalInclTax(),
                    'row_total_discount' => (float)$productItem->getDiscountAmount()
                ];
                if ($configAttributes && $attributes->hasValues()) {
                    $productData['product_attributes'] = $attributes;
                }
                if ($customOptions) {
                    $productData['custom_fields'] = $customOptions;
                }
                $this->order_lines[] = $productData;
            } else {
                // when no product information is available limit to this data
                $productData = [
                    'id' => $productItem->getProductId(),
                    'order_id' => $order_id,
                    'order_line_id' => $productItem->getItemId(),
                    'product_sku' => $productItem->getSku(),
                    'product_name' => $productItem->getName(),
                    'product_description' => $productItem->getDescription(),
                    'tax_percent' => (float)$productItem->getTaxPercent(),
                    'product_price' => (float)$productItem->getPrice(),
                    'product_price_tax' => (float)$productItem->getTaxAmount(),
                    'product_price_inc_tax' => (float)$productItem->getPriceInclTax(),
                    'product_original_price' => (float)$productItem->getBasePrice(),
                    'quantity' => (float)number_format(
                        $productItem->getData('qty_ordered'),
                        2
                    ),
                    'row_total' => (float)$productItem->getRowTotal(),
                    'row_total_inc_tax' => (float)$productItem->getRowTotalInclTax(),
                    'row_total_discount' => (float)$productItem->getDiscountAmount()
                ];
                if (!$customOptions) {
                    unset($productData['custom-options']);
                }
                $this->order_lines[] = $productData;
            }
        }
    }

    /**
     * Get the street name by line number.
     *
     * @param string  $street
     * @param integer $line
     *
     * @return string
     */
    public function _getStreet($street, $line)
    {
        $street = explode("\n", $street);
        if ($line == 1) {
            return $street[0];
        }
        if (isset($street[$line - 1])) {
            return $street[$line - 1];
        } else {
            return '';
        }
    }

    /**
     * Exposes the class as an array of objects.
     * Return any exposed data that will included into the import as transactional data for Orders.
     *
     * @return array
     */
    public function expose()
    {
        $properties = array_diff_key(
            get_object_vars($this),
            array_flip([
                'storeManager',
                'helper',
                'customerFactory',
                'productFactory',
                'attributeCollection',
                'attributeSet',
                'productResource',
                'attributeHandler',
                'validator'
            ])
        );
        //remove null/0/false values
        $properties = array_filter($properties);

        return $properties;
    }

    /**
     * Get attribute value for the field.
     *
     * @param array                      $field
     * @param \Magento\Sales\Model\Order $orderData
     *
     * @return float|integer|null|string
     */
    public function _getCustomAttributeValue($field, $orderData)
    {
        $type = $field['DATA_TYPE'];

        $function = 'get';
        $exploded = explode('_', $field['COLUMN_NAME']);
        foreach ($exploded as $one) {
            $function .= ucfirst($one);
        }

        $value = null;
        try {
            switch ($type) {
                case 'int':
                case 'smallint':
                    $value = (int)$orderData->$function();
                    break;

                case 'decimal':
                    $value = (float)number_format(
                        $orderData->$function(),
                        2,
                        '.',
                        ''
                    );
                    break;

                case 'timestamp':
                case 'datetime':
                case 'date':
                    $date = new \DateTime($orderData->$function());
                    $value = $date->format(\DateTime::ISO8601);
                    break;

                default:
                    $value = $orderData->$function();
            }
        } catch (\Exception $e) {
            $this->helper->debug((string)$e, []);
        }

        return $value;
    }

    /**
     * Create property on runtime.
     *
     * @param string $field
     * @param mixed  $value
     *
     * @return null
     */
    public function _assignCustom($field, $value)
    {
        $this->custom[$field['COLUMN_NAME']] = $value;
    }

    /**
     * @param \Magento\Sales\Model\Order\Item $orderItem
     * @return array
     */
    public function _getOrderItemOptions($orderItem)
    {
        $orderItemOptions = $orderItem->getProductOptions();

        //if product doesn't have options
        if (!array_key_exists('options', $orderItemOptions)) {
            return [];
        }

        $orderItemOptions = $orderItemOptions['options'];

        //if product options isn't array
        if (!is_array($orderItemOptions)) {
            return [];
        }

        $options = [];

        foreach ($orderItemOptions as $orderItemOption) {
            if (array_key_exists('value', $orderItemOption)
                && array_key_exists(
                    'label',
                    $orderItemOption
                )
            ) {
                $label = $this->validator->cleanLabel(
                    $orderItemOption['label'],
                    '-',
                    $orderItemOption['option_id']
                );
                if (empty($label)) {
                    continue;
                }
                $options[][$label] = $orderItemOption['value'];
            }
        }

        return $options;
    }
}
