<?php

namespace Pinpoint\Conesso\Model\Customer;

/**
 * Guest sync cronjob.
 */
class Guest
{
    /**
     * @var \Pinpoint\Conesso\Model\ResourceModel\Contact
     */
    private $contactResource;

    /**
     * @var integer
     */
    private $countGuests = 0;

    /**
     * @var mixed
     */
    private $start;

    /**
     * @var \Pinpoint\Conesso\Helper\Data
     */
    private $helper;

    /**
     * @var \Pinpoint\Conesso\Helper\File
     */
    private $file;

    /**
     * @var \Pinpoint\Conesso\Model\ContactFactory
     */
    private $contactFactory;

    /**
     * @var \Pinpoint\Conesso\Model\ImporterFactory
     */
    private $importerFactory;

    /**
     * Guest constructor.
     * @param \Pinpoint\Conesso\Model\ImporterFactory       $importerFactory
     * @param \Pinpoint\Conesso\Model\ContactFactory        $contactFactory
     * @param \Pinpoint\Conesso\Helper\File                 $file
     * @param \Pinpoint\Conesso\Model\ResourceModel\Contact $contactResource
     * @param \Pinpoint\Conesso\Helper\Data                 $helper
     */
    public function __construct(
        \Pinpoint\Conesso\Model\ImporterFactory $importerFactory,
        \Pinpoint\Conesso\Model\ContactFactory $contactFactory,
        \Pinpoint\Conesso\Helper\File $file,
        \Pinpoint\Conesso\Model\ResourceModel\Contact $contactResource,
        \Pinpoint\Conesso\Helper\Data $helper
    ) {
        $this->importerFactory = $importerFactory;
        $this->contactFactory = $contactFactory;
        $this->contactResource = $contactResource;
        $this->helper = $helper;
        $this->file = $file;
    }

    /**
     * GUEST SYNC.
     *
     * @return null
     */
    public function sync()
    {
        $this->start = microtime(true);
        $websites    = $this->helper->getWebsites();

        foreach ($websites as $website) {
            //check if the guest is mapped and enabled
            $addresbook = $this->helper->getGuestAddressBook($website);
            $guestSyncEnabled = $this->helper->isGuestSyncEnabled($website);
            $apiEnabled = $this->helper->isEnabled($website);
            if ($addresbook && $guestSyncEnabled && $apiEnabled) {
                //sync guests for website
                $this->exportGuestPerWebsite($website);
            }
        }
        if ($this->countGuests) {
            $this->helper->log(
                '----------- Guest sync ----------- : ' .
                gmdate('H:i:s', microtime(true) - $this->start) .
                ', Total synced = ' . $this->countGuests
            );
        }
    }

    /**
     * Export guests for a website.
     *
     * @param \Magento\Store\Model\Website $website
     *
     * @return null
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function exportGuestPerWebsite($website)
    {
        $onlySubscribers = $this->helper->isOnlySubscribersForContactSync($website->getWebsiteId());
        $contact = $this->contactFactory->create();
        $guests = ($onlySubscribers) ? $contact->getGuests($website, true) :
            $contact->getGuests($website);

        //found some guests
        if ($guests->getSize()) {
            $guestFilename = strtolower(
                $website->getCode() . '_guest_'
                . date('d_m_Y_His') . '.csv'
            );
            $this->helper->log('Guest file: ' . $guestFilename);
            $storeName = $this->helper->getMappedStoreName($website);
            $this->file->outputCSV(
                $this->file->getFilePath($guestFilename),
                ['Email', 'emailType', $storeName]
            );

            foreach ($guests as $guest) {
                $this->outputCsvToFile($guest, $website, $guestFilename);
            }
            if ($this->countGuests) {
                //register in queue with importer
                $this->importerFactory->create()
                    ->registerQueue(
                        \Pinpoint\Conesso\Model\Importer::IMPORT_TYPE_GUEST,
                        '',
                        \Pinpoint\Conesso\Model\Importer::MODE_BULK,
                        $website->getId(),
                        $guestFilename
                    );
            }
        }
    }

    /**
     * Output
     *
     * @param \Pinpoint\Conesso\Model\Contact $guest
     * @param \Magento\Store\Model\Website      $website
     * @param string                            $guestFilename
     *
     * @return null
     */
    private function outputCsvToFile($guest, $website, $guestFilename)
    {
        $email = $guest->getEmail();
        $guest->setEmailImported(\Pinpoint\Conesso\Model\Contact::EMAIL_CONTACT_IMPORTED);

        $this->contactResource->save($guest);
        $storeName = $website->getName();
        // save data for guests
        $this->file->outputCSV(
            $this->file->getFilePath($guestFilename),
            [$email, 'Html', $storeName]
        );
        ++$this->countGuests;
    }
}
