<?php

namespace Pinpoint\Conesso\Model\Sync;

use Magento\Store\Api\StoreRepositoryInterface;
use Pinpoint\Conesso\Helper\Config;
use Pinpoint\Conesso\Model\Importer as ImporterModel;

class Importer implements SyncInterface
{
    //sync limits
    const SYNC_SINGLE_LIMIT_NUMBER = 100;

    /**
     * @var \Pinpoint\Conesso\Helper\Data
     */
    private $helper;

    /**
     * @var \Pinpoint\Conesso\Helper\File
     */
    private $fileHelper;

    /**
     * @var \Pinpoint\Conesso\Model\ImporterFactory
     */
    private $importerFactory;

    /**
     * @var \Magento\Framework\ObjectManagerInterface
     */
    private $objectManager;

    /**
     * @var \Magento\Framework\Stdlib\DateTime
     */
    private $dateTime;

    /**
     * @var StoreRepositoryInterface $storeRepository
     */
    private $storeRepository;

    /**
     * @var array
     */
    private $reasons
        = [
            'Globally Suppressed',
            'Blocked',
            'Unsubscribed',
            'Hard Bounced',
            'Isp Complaints',
            'Domain Suppressed',
            'Failures',
            'Invalid Entries',
            'Mail Blocked',
            'Suppressed by you',
        ];

    /**
     * @var array
     */
    private $importStatuses
        = [
            'RejectedByWatchdog',
            'InvalidFileFormat',
            'Unknown',
            'Failed',
            'ExceedsAllowedContactLimit',
            'NotAvailableInThisVersion',
        ];

    /**
     * @var array
     */
    private $bulkPriority;

    /**
     * @var array
     */
    private $singlePriority;

    /**
     * @var integer
     */
    private $totalItems;

    /**
     * @var integer
     */
    private $bulkSyncLimit;

    /**
     * Importer constructor.
     *
     * @param \Pinpoint\Conesso\Helper\Data           $helper
     * @param \Pinpoint\Conesso\Helper\File           $fileHelper
     * @param \Pinpoint\Conesso\Model\ImporterFactory $importerFactory
     * @param \Magento\Framework\ObjectManagerInterface $objectManager
     * @param \Magento\Framework\Stdlib\DateTime        $dateTime
     * @param StoreRepositoryInterface                  $storeRepository
     */
    public function __construct(
        \Pinpoint\Conesso\Helper\Data $helper,
        \Pinpoint\Conesso\Helper\File $fileHelper,
        \Pinpoint\Conesso\Model\ImporterFactory $importerFactory,
        \Magento\Framework\ObjectManagerInterface $objectManager,
        \Magento\Framework\Stdlib\DateTime $dateTime,
        StoreRepositoryInterface $storeRepository
    ) {
        $this->helper = $helper;
        $this->fileHelper = $fileHelper;
        $this->importerFactory = $importerFactory;
        $this->objectManager = $objectManager;
        $this->dateTime = $dateTime;
        $this->storeRepository = $storeRepository;
    }

    /**
     * Importer sync
     */
    public function sync(\DateTime $from = null)
    {
        $this->processQueue();
    }

    /**
     * Process the data from queue.
     *
     * @return null
     * @throws \Magento\Framework\Exception\AlreadyExistsException
     */
    private function processQueue()
    {
        //Set items to 0
        $this->totalItems = 0;

        //Set bulk sync limit
        $this->bulkSyncLimit = 5;

        //Set priority
        $this->_setPriority();

        $enabledWebsites = $this->getECEnabledWebsites();

        //Check previous import status
        $this->_checkImportStatus();

        $importerModel = $this->importerFactory->create();

        //Bulk priority. Process group 1 first
        foreach ($this->bulkPriority as $bulk) {
            if ($this->totalItems < $bulk['limit']) {
                $collection = $importerModel->_getQueue(
                    $bulk['type'],
                    $bulk['mode'],
                    $bulk['limit'] - $this->totalItems,
                    $enabledWebsites
                );
                if ($collection->getSize()) {
                    $this->totalItems += $collection->getSize();
                    $bulkModel = $this->objectManager->create($bulk['model']);
                    $bulkModel->sync($collection);
                }
            }
        }

        //reset total items to 0
        $this->totalItems = 0;

        //Single/Update priority.
        foreach ($this->singlePriority as $single) {
            if ($this->totalItems < $single['limit']) {
                $collection = $importerModel->_getQueue(
                    $single['type'],
                    $single['mode'],
                    $single['limit'] - $this->totalItems,
                    $enabledWebsites
                );
                if ($collection->getSize()) {
                    $this->totalItems += $collection->getSize();
                    $singleModel = $this->objectManager->create(
                        $single['model']
                    );
                    $singleModel->sync($collection);
                }
            }
        }
    }

    /**
     * Set importing priority.
     *
     * @return null
     *
     * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
     */
    private function _setPriority()
    {
        /*
         * Bulk
         */

        $defaultBulk = [
            'model' => '',
            'mode' => ImporterModel::MODE_BULK,
            'type' => '',
            'limit' => $this->bulkSyncLimit,
        ];

        //Contact Bulk
        $contact = $defaultBulk;
        $contact['model'] = \Pinpoint\Conesso\Model\Sync\Contact\Bulk::class;
        $contact['type'] = [
            ImporterModel::IMPORT_TYPE_CONTACT,
            ImporterModel::IMPORT_TYPE_GUEST,
            ImporterModel::IMPORT_TYPE_SUBSCRIBERS,
        ];

        //Bulk Order
        $order = $defaultBulk;
        $order['model'] = \Pinpoint\Conesso\Model\Sync\Td\Bulk::class;
        $order['type'] = ImporterModel::IMPORT_TYPE_ORDERS;

        //Bulk Other TD
        $other = $defaultBulk;
        $other['model'] = \Pinpoint\Conesso\Model\Sync\Td\Bulk::class;
        $other['type'] = [
            'Catalog',
            ImporterModel::IMPORT_TYPE_REVIEWS,
            ImporterModel::IMPORT_TYPE_WISHLIST,
        ];

        /*
         * Update
         */
        $defaultSingleUpdate = [
            'model' => \Pinpoint\Conesso\Model\Sync\Contact\Update::class,
            'mode' => '',
            'type' => '',
            'limit' => self::SYNC_SINGLE_LIMIT_NUMBER,
        ];

        //Subscriber resubscribe
        $subscriberResubscribe = $defaultSingleUpdate;
        $subscriberResubscribe['mode'] = ImporterModel::MODE_SUBSCRIBER_RESUBSCRIBED;
        $subscriberResubscribe['type'] = ImporterModel::IMPORT_TYPE_SUBSCRIBER_RESUBSCRIBED;

        //Subscriber update/suppressed
        $subscriberUpdate = $defaultSingleUpdate;
        $subscriberUpdate['mode'] = ImporterModel::MODE_SUBSCRIBER_UPDATE;
        $subscriberUpdate['type'] = ImporterModel::IMPORT_TYPE_SUBSCRIBER_UPDATE;

        //Email Change
        $emailChange = $defaultSingleUpdate;
        $emailChange['mode'] = ImporterModel::MODE_CONTACT_EMAIL_UPDATE;
        $emailChange['type'] = ImporterModel::IMPORT_TYPE_CONTACT_UPDATE;

        //Order Update
        $orderUpdate = $defaultSingleUpdate;
        $orderUpdate['model'] = \Pinpoint\Conesso\Model\Sync\Td\Update::class;
        $orderUpdate['mode'] = ImporterModel::MODE_SINGLE;
        $orderUpdate['type'] = ImporterModel::IMPORT_TYPE_ORDERS;

        //CartInsight TD update
        $updateCartInsightTd = $defaultSingleUpdate;
        $updateCartInsightTd['model'] = \Pinpoint\Conesso\Model\Sync\Td\Update::class;
        $updateCartInsightTd['mode'] = ImporterModel::MODE_SINGLE;
        $updateCartInsightTd['type'] = ImporterModel::IMPORT_TYPE_CART_INSIGHT_CART_PHASE;

        //Update Other TD
        $updateOtherTd = $defaultSingleUpdate;
        $updateOtherTd['model'] = \Pinpoint\Conesso\Model\Sync\Td\Update::class;
        $updateOtherTd['mode'] = ImporterModel::MODE_SINGLE;
        $updateOtherTd['type'] = [
            'Catalog',
            ImporterModel::IMPORT_TYPE_WISHLIST,
        ];

        /*
        * Delete
        */
        $defaultSingleDelete = [
            'model' => '',
            'mode' => '',
            'type' => '',
            'limit' => self::SYNC_SINGLE_LIMIT_NUMBER,
        ];

        //Contact Delete
        $contactDelete = $defaultSingleDelete;
        $contactDelete['model'] = \Pinpoint\Conesso\Model\Sync\Contact\Delete::class;
        $contactDelete['mode'] = ImporterModel::MODE_CONTACT_DELETE;
        $contactDelete['type'] = ImporterModel::IMPORT_TYPE_CONTACT;

        //TD Delete
        $tdDelete = $defaultSingleDelete;
        $tdDelete['model'] = \Pinpoint\Conesso\Model\Sync\Td\Delete::class;
        $tdDelete['mode'] = ImporterModel::MODE_SINGLE_DELETE;
        $tdDelete['type'] = [
            'Catalog',
            ImporterModel::IMPORT_TYPE_REVIEWS,
            ImporterModel::IMPORT_TYPE_WISHLIST,
            ImporterModel::IMPORT_TYPE_ORDERS,
        ];

        //Bulk Priority
        $this->bulkPriority = [
            $contact,
            $order,
            $other,
        ];

        $this->singlePriority = [
            $subscriberResubscribe,
            $subscriberUpdate,
            $emailChange,
            $orderUpdate,
            $updateCartInsightTd,
            $updateOtherTd,
            $contactDelete,
            $tdDelete,
        ];
    }

    /**
     * Check importing status for pending import.
     *
     * @return null
     * @throws \Magento\Framework\Exception\AlreadyExistsException
     */
    private function _checkImportStatus()
    {
        $importerModel = $this->importerFactory->create();
        if ($items = $importerModel->_getImportingItems($this->bulkSyncLimit)) {
            foreach ($items as $item) {
                $websiteId = $item->getWebsiteId();
                $client = false;
                if ($this->helper->isEnabled($websiteId)) {
                    $client = $this->helper->getWebsiteApiClient(
                        $websiteId
                    );
                }
                if (!$client) {
                    $item->setMessage('No API client found for this website.')
                        ->setImportStatus(ImporterModel::FAILED);
                        $importerModel->saveItem($item);
                        continue;
                }

                try {
                    if ($item->getImportType() == ImporterModel::IMPORT_TYPE_CONTACT ||
                        $item->getImportType() == ImporterModel::IMPORT_TYPE_SUBSCRIBERS ||
                        $item->getImportType() == ImporterModel::IMPORT_TYPE_GUEST
                    ) {
                        $response = $client->getContactsImportByImportId($item->getImportId());
                    } else {
                        $response = $client->getContactsTransactionalDataImportByImportId(
                            $item->getImportId()
                        );
                    }
                } catch (\Exception $e) {
                    $item->setMessage($e->getMessage())
                        ->setImportStatus(ImporterModel::FAILED);
                    $importerModel->saveItem($item);
                    continue;
                }

                $this->processResponse($response, $item, $websiteId);
            }
        }
    }

    /**
     * @param object                             $response
     * @param \Pinpoint\Conesso\Model\Importer $item
     * @param integer                            $websiteId
     *
     * @return null
     */
    private function processResponse($response, $item, $websiteId)
    {
        if (isset($response->message)) {
            $item->setImportStatus(ImporterModel::FAILED)
                ->setMessage($response->message);
        } else {
            if ($response->status == 'Finished') {
                $item = $this->processFinishedItem($item, $websiteId);
            } elseif (in_array($response->status, $this->importStatuses)) {
                $item->setImportStatus(ImporterModel::FAILED)
                    ->setMessage('Import failed with status ' . $response->status);
            } else {
                //Not finished
                $this->totalItems += 1;
            }
        }
        //Save item
        $this->importerFactory->create()
            ->saveItem($item);
    }

    /**
     * @param \Pinpoint\Conesso\Model\Importer $item
     * @param integer                            $websiteId
     *
     * @return $this
     *
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    private function processFinishedItem($item, $websiteId)
    {
        $now = gmdate('Y-m-d H:i:s');

        $item->setImportStatus(ImporterModel::IMPORTED)
            ->setImportFinished($now)
            ->setMessage('');

        if ($item->getImportType() == ImporterModel::IMPORT_TYPE_CONTACT ||
            $item->getImportType() == ImporterModel::IMPORT_TYPE_SUBSCRIBERS ||
            $item->getImportType() == ImporterModel::IMPORT_TYPE_GUEST
        ) {
            $file = $item->getImportFile();
            // if a filename is stored in the table and if that file physically exists
            if ($file && $this->fileHelper->isFilePathExistWithFallback($file)) {
                //remove the consent data for contacts before archiving the file
                $log = $this->fileHelper->cleanProcessedConsent(
                    $this->fileHelper->getFilePathWithFallback($file)
                );
                if ($log) {
                    $this->helper->log($log);
                }
                if (! $this->fileHelper->isFileAlreadyArchived($file)) {
                    $this->fileHelper->archiveCSV($file);
                }
            }

            if ($item->getImportId()) {
                $this->_processContactImportReportFaults($item->getImportId(), $websiteId);
            }
        }

        return $item;
    }

    /**
     * Get report info for contacts sync.
     *
     * @param integer $id
     * @param integer $websiteId
     *
     * @throws \Magento\Framework\Exception\LocalizedException
     *
     * @return null
     */
    private function _processContactImportReportFaults($id, $websiteId)
    {
        $client = $this->helper->getWebsiteApiClient($websiteId);
        $report = $client->getContactImportReportFaults($id);

        if ($report) {
            $reportData = explode(PHP_EOL, $this->_removeUtf8Bom($report));
            //unset header
            unset($reportData[0]);
            //no data in report
            if (! empty($reportData)) {
                $contacts = [];
                foreach ($reportData as $row) {
                    if (empty($row)) {
                        continue;
                    }

                    list ($reason, $email) = explode(',', $row);
                    if (in_array($reason, $this->reasons)) {
                        $contacts[] = $email;
                    }
                }

                // get a time period for the last contact sync
                $cronMinutes = filter_var($this->helper->getConfigValue(Config::XML_PATH_CRON_SCHEDULE_CONTACT), FILTER_SANITIZE_NUMBER_INT);
                $lastSyncPeriod = new \DateTime($this->dateTime->formatDate(true), new \DateTimeZone('UTC'));
                $lastSyncPeriod->sub(new \DateInterval("PT{$cronMinutes}M"));

                // check whether any last subscribe dates fall within the last subscriber sync period
                $recentlyResubscribed = array_filter(
                    $this->helper->contactResource->getLastSubscribedAtDates($contacts),
                    function ($contact) use ($lastSyncPeriod) {
                        $lastSubscribed = new \DateTime($contact['last_subscribed_at'], new \DateTimeZone('UTC'));
                        return $lastSubscribed >= $lastSyncPeriod;
                    }
                );

                if (!empty($recentlyResubscribed)) {
                    $importerModel = $this->importerFactory->create();
                    // queue resubscription jobs
                    foreach ($recentlyResubscribed as $resubscriber) {
                        $importerModel->registerQueue(
                            \Pinpoint\Conesso\Model\Importer::IMPORT_TYPE_SUBSCRIBER_RESUBSCRIBED,
                            ['email' => $resubscriber['email']],
                            \Pinpoint\Conesso\Model\Importer::MODE_SUBSCRIBER_RESUBSCRIBED,
                            $websiteId
                        );
                    }

                    // remove from unsubscribable emails
                    $contacts = array_diff($contacts, array_column($recentlyResubscribed, 'email'));
                }

                //unsubscribe from email contact and newsletter subscriber tables
                if (!empty($contacts)) {
                    $this->helper->contactResource->unsubscribe($contacts);
                }
            }
        }
    }

    /**
     * Convert utf8 data.
     *
     * @param string $text
     *
     * @return string
     */
    private function _removeUtf8Bom($text)
    {
        $bom = pack('H*', 'EFBBBF');
        $text = preg_replace("/^$bom/", '', $text);

        return $text;
    }

    /**
     * Retrieve a list of websites that have Engagement Cloud enabled.
     *
     * @return array
     */
    private function getECEnabledWebsites()
    {
        $stores = $this->storeRepository->getList();
        $websiteIds = [];
        foreach ($stores as $store) {
            $websiteId = $store->getWebsiteId();
            if ($this->helper->isEnabled($websiteId)
                && !in_array($websiteId, $websiteIds)
            ) {
                $websiteIds[] = $websiteId;
            }
        }
        return $websiteIds;
    }
}
